'use client'
import { createIbcRequest, deleteIbc, editIbcRequest } from "@/app/lib/ibc/action";
import DataTable from "@/components/tables/DataTable";
import { DeleteIcon, EditIcon } from "@chakra-ui/icons";
import { Button, Flex, Icon, useDisclosure, useToast } from "@chakra-ui/react";
import { useRouter } from "next/navigation";
import { useState } from "react";
import { CycleDaysInterface } from "../type/cycle";
import { columnsDataBlocks } from "../variables/columnsBlock";
import IbcModalForm from "./form";
import { createCycleDaysRequest, deleteCycleDays, editCycleDaysRequest } from "@/app/lib/cycle-days/action";
import { useGeneral } from "@/contexts/GeneralContext";
import { hasPermission } from "@/utils/navigation";

interface CycleDaysTableProps {
  data: CycleDaysInterface[];
}

const CycleDaysTable: React.FC<CycleDaysTableProps> = ({ data }) => {
  const { currentUser: currentUserGlobal } = useGeneral();
  const { isOpen, onOpen, onClose } = useDisclosure();
  const [currentUser, setCurrentUser] = useState<CycleDaysInterface | null>(null);
  const toast = useToast();
  const router = useRouter();

  const deleteUser = async (id: number) => {
    const res = await deleteCycleDays(id);
    if (res?.status == 200) {
      toast({
        title: "Success",
        description: res.message,
        status: "success",
        position: "top-right",
        duration: 2000,
        isClosable: true,
      });
      router.refresh();
    } else {
      toast({
        title: "Error",
        description: res.message,
        status: "error",
        position: "top-right",
        duration: 9000,
        isClosable: true,
      });
    }
  };

  const handleAddClick = () => {
    setCurrentUser(null); // Reset for add
    onOpen();
  };

  const handleEditClick = (id: number) => {
    const singleUser = data?.data?.find((item: CycleDaysInterface) => (item.id === id));
    setCurrentUser((prev) => {
      return { ...singleUser };
    });
    onOpen();
  };

  const handleSubmit = async (values: CycleDaysInterface) => {
    if (currentUser) {
      const payload = { userId: currentUser.id, data: values };
      const res = await editCycleDaysRequest(payload);
      if (res?.status == 200) {
        toast({
          title: "Success",
          description: res.message,
          status: "success",
          position: "top-right",
          duration: 2000,
          isClosable: true,
        });
        onClose();
        router.refresh();
      } else {
        toast({
          title: "Error",
          description: res.message,
          status: "error",
          position: "top-right",
          duration: 9000,
          isClosable: true,
        });
      }
    } else {
      const res = await createCycleDaysRequest(values);
      if (res?.status == 200) {
        toast({
          title: "Success",
          description: res.message,
          status: "success",
          position: "top-right",
          duration: 2000,
          isClosable: true,
        });
        onClose();
        router.refresh();
      } else {
        toast({
          title: "Error",
          description: res.message,
          status: "error",
          position: "top-right",
          duration: 9000,
          isClosable: true,
        });
      }
    }
  };

  const action = (user: number) => (
    <Flex gap={2}>
      {hasPermission(currentUserGlobal.permissions, "setting.cycle-day", "edit") && (
        <Button
          variant="blank"
          onClick={() => handleEditClick(user)}
          w="37px"
          h="37px"
          borderRadius="10px">
          <Icon as={EditIcon} color="brand.500" w="20px" h="20px" />
        </Button>
      )}
      {hasPermission(currentUserGlobal.permissions, "setting.cycle-day", "delete") && (
        <Button
          variant="blank"
          onClick={() => deleteUser(user)}
          w="37px"
          h="37px"
          borderRadius="10px">
          <Icon as={DeleteIcon} color="brand.500" w="20px" h="20px" />
        </Button>
      )}
    </Flex>
  );

  return (
    <>
      <DataTable
        addButton={hasPermission(currentUserGlobal.permissions, "setting.cycle-day", "create") && <Button variant="solid" colorScheme="blue" onClick={handleAddClick}>Create</Button>}
        columnsData={columnsDataBlocks}
        tableData={data?.data as unknown as CycleDaysInterface[]}
        paginate={data}
        isSelectable={false}
        showPageSize={false}
        actionSlot={(user: number) => action(user)}
      />

      <IbcModalForm
        isOpen={isOpen}
        onClose={onClose}
        currentUser={currentUser}
        handleSubmit={handleSubmit}
      />
    </>
  );
};

export default CycleDaysTable;
